<?php

namespace Drupal\Tests\telephone_international_widget\FunctionalJavascript;

use Drupal\Core\Entity\Entity\EntityViewDisplay;
use Drupal\node\Entity\NodeType;
use Drupal\field\Entity\FieldConfig;
use Drupal\field\Entity\FieldStorageConfig;
use Drupal\Core\Entity\Entity\EntityFormDisplay;
use Drupal\Core\Field\FieldStorageDefinitionInterface;
use Drupal\FunctionalJavascriptTests\WebDriverTestBase;

/**
 * Tests the telephone_international widget.
 *
 * @group telephone_international_widget
 */
class TelephoneInternationalWidgetBehaviorTest extends WebDriverTestBase {

  /**
   * Modules to enable.
   *
   * @var string[]
   */
  protected static $modules = [
    'node',
    'field',
    'telephone',
    'telephone_international_widget',
  ];

  /**
   * A simple theme for rendering forms.
   *
   * @var string
   */
  protected $defaultTheme = 'stark';

  /**
   * Set up a test content type.
   *
   * Include an unlimited-cardinality telephone field using our international
   * widget.
   */
  protected function setUp(): void {
    parent::setUp();

    // 1) Create a content type.
    NodeType::create([
      'type' => 'test_type',
      'name' => 'Test Type',
    ])->save();

    $this->drupalLogin($this->drupalCreateUser([
      'administer filters',
      'create test_type content',
      'edit own test_type content',
    ]));

    // 2) Define unlimited storage for field_phone.
    FieldStorageConfig::create([
      'field_name'  => 'field_phone',
      'entity_type' => 'node',
      'type'        => 'telephone',
      'cardinality' => FieldStorageDefinitionInterface::CARDINALITY_UNLIMITED,
    ])->save();

    // 3) Attach it to our bundle.
    FieldConfig::create([
      'field_name'  => 'field_phone',
      'entity_type' => 'node',
      'bundle'      => 'test_type',
      'label'       => 'Phone',
    ])->save();

    // 4) Override the form display to use our widget.
    EntityFormDisplay::create([
      'targetEntityType' => 'node',
      'bundle'           => 'test_type',
      'mode'             => 'default',
      'status'           => TRUE,
    ])
      ->setComponent('field_phone', [
        'type' => 'telephone_international',
      ])
      ->save();

    // 5) Define the view display.
    EntityViewDisplay::create(
      [
        'targetEntityType' => 'node',
        'bundle'           => 'test_type',
        'mode'             => 'default',
        'status'           => TRUE,
      ]
    )->setComponent('field_phone', [
      'type' => 'string',
      'label' => 'hidden',
    ])
      ->save();
  }

  /**
   * @covers ::formElement
   * @group telephone_international_widget
   *
   * Ensure that the telephone field is visible.
   */
  public function testTelephoneFieldIsVisible(): void {
    $page = $this->getSession()->getPage();
    $assert_session = $this->assertSession();

    $this->drupalGet('/node/add/test_type');

    $this->assertTrue($page->hasField('field_phone[0][value]'));

    $this->assertNotNull($assert_session->waitForElement('css', '.iti__flag-container'));
  }

  /**
   * @covers ::formElement
   * @group telephone_international_widget
   *
   * Ensure that the telephone field is applied to multiple fields.
   */
  public function testTelephoneField(): void {
    $page = $this->getSession()->getPage();
    $assert = $this->assertSession();

    // Load the add-node form.
    $this->drupalGet('/node/add/test_type');

    // Make sure there’s exactly one telephone widget to start.
    $initial = $page->findAll('css', 'input.telephone_international_widget');
    $this->assertCount(1, $initial, 'One telephone widget at load.');

    // Click the "Add another item" button.
    $page->pressButton('Add another item');

    // Wait for the new widget (delta 1) to appear.
    $assert->waitForElementVisible(
      'css',
      'input[name="field_phone[1][value]"]'
    );

    // Re-count the visible widgets.
    $after = $page->findAll('css', 'input.telephone_international_widget');
    $this->assertCount(2, $after, 'Two telephone widgets after clicking Add another item.');

    // Ensure that values are saved properly.
    $edit = [
      'title[0][value]' => 'Test Phone',
      'field_phone[0][value]' => '+1 202 555 0100',
      'field_phone[1][value]' => '+34666999333',
    ];

    $this->submitForm($edit, 'Save');

    $this->assertSession()->statusMessageExists('status');
    $this->assertSession()->pageTextContains('Test Phone');
    $this->assertSession()->pageTextContains('+12025550100');
    $this->assertSession()->pageTextContains('+34666999333');

    // Check that values are preserved when resaving unchanged form.
    $node = $this->drupalGetNodeByTitle('Test Phone');
    $this->drupalGet('node/' . $node->id() . '/edit');
    $this->assertSession()->waitForElement('css', '.iti__flag-container');

    $this->click('#edit-submit');
    $this->assertSession()->statusMessageExists('status');
    $this->assertSession()->pageTextContains('Test Phone');
    $this->assertSession()->pageTextContains('+12025550100');
    $this->assertSession()->pageTextContains('+34666999333');

  }

}
